local T = { }

-- target options
local boards = {'custom', 'launchpad', 'controlcard'} -- must match info.xml file
local uniFlashConfigs = {
    launchpad = 'Launchpad_TMS320F280049C.ccxml',
    controlcard = 'ControlCard_TMS320F280049C.ccxml'
}

function T.GetBoardNameFromComboIndex(index)
    return boards[index]
end

function T.GetUniflashConfig(board)
    return uniFlashConfigs[board]
end

-- resources
local pwmGpio = {}
pwmGpio[1] = {0,1}
pwmGpio[2] = {2,3}
pwmGpio[3] = {4,5}
pwmGpio[4] = {6,7}
pwmGpio[5] = {8,9}
pwmGpio[6] = {10,11}    
pwmGpio[7] = {12,13}
pwmGpio[8] = {14,15}  

local qepPinset = {}
qepPinset["28-29-59"] = 10
qepPinset["35-37-59"] = 11

qepPinset["14-15-26"] = 20

local spiPinset = {}
spiPinset["16-17-56-57"] = 10
spiPinset["24-31-22-33"] = 20

local sciPinset = {}
sciPinset["28-29"] = 0
sciPinset["35-37"] = 1

function T.Configure(resources, registry)
  resources:add("Model Trigger", 1, 1)  
  resources:add("Powerstage Control", 1, 1)  	
  resources:add("GPIO", 0, 59)    
  resources:add("ADC", 0, 2)   
  resources:add("PWM", 1, 8) 
  resources:add("QEP", 1, 2)  
  resources:add("SPI", 1, 2)
  resources:add("ADCA-SOC", 0, 15)
  resources:add("ADCB-SOC", 0, 15)  
  resources:add("ADCC-SOC", 0, 15)    
end

function T.GetPwmGpio(unit)
	if pwmGpio[unit]==nil then
	  return -1 -- Error code - resource not available.
	else
	  return pwmGpio[unit]
	end
end

function T.GetSciPinset(pinsetS)
	return sciPinset[pinsetS]
end

function T.GetQepPinset(pinsetS)
	return qepPinset[pinsetS]
end

function T.GetSpiPinset(pinsetS)
	return spiPinset[pinsetS]
end

function T.GetIntOscClock()
	return 10000000 -- INTOSC2
end

function T.GetTimerClock()
    -- to help with margin and latency measurements, timer is configured at rate of PWM
	return T.GetPwmClock()
end

function T.GetPwmClock()
	-- TBCLOCK - CLKDIV=/1, HSPCLKDIV=/2
	local sysClkHz = Target.Variables.sysClkMHz * 1e6
	return sysClkHz / 2
end

function T.GetDeadTimeClock()
	-- TBCLOCK - CLKDIV=/1, HSPCLKDIV=/2 
	-- TBCLK = EPWMCLK/(HSPCLKDIV * CLKDIV)
	local sysClkHz = Target.Variables.sysClkMHz * 1e6
	return sysClkHz / 2 
end

function T.GetMaxEventPeriod()
	return 15
end

function T.GetAdcClock()
	-- HAL automatically configures ADCCTL2.bit.PRESCALE
	local sysClkHz = Target.Variables.sysClkMHz * 1e6
	if sysClkHz <= 50000000 then
		return sysClkHz
	else
		return sysClkHz / 2
	end
end

function T.GetMaxSciBaudRate()
	-- assuming 8N1. 1.5 characters per poll
	local maxRate = 1/Target.Variables.SAMPLE_TIME * 15
	return math.min(115200, maxRate)
end

function T.CalcACQPS(ts)
	local tsMin = math.max(75e-9, 1/T.GetAdcClock())-- per datasheet
	if ts ~= ts or ts < tsMin then -- test for NAN or value that is too small
		ts = tsMin 
	end
	local sysClkHz = Target.Variables.sysClkMHz * 1e6
	return math.floor(ts * sysClkHz)
end

function T.GetINTSEL1N2RegisterName()
	return "ADCINTSEL1N2"
end

function T.WriteClockConfiguration(io)
  local sysClkHz = Target.Variables.sysClkMHz * 1e6
	
  local clkin, clksrc
  
  if sysClkHz > 100000000 then
    return "Excessive system clock setting."
  end 
  
  if sysClkHz ~= math.floor(sysClkHz) then
    return "System clock setting must be integer value."
  end 

  if Target.Variables.useIntOsc ~= 1 then
  	clkin = Target.Variables.extClkMHz * 1e6
  	clksrc = 1
  else
  	clkin =  T.GetIntOscClock()
  	clksrc = 0
  end

  local imult = math.min(math.floor(2*sysClkHz/clkin), 127)
  if not (sysClkHz == imult * clkin/2) then
    return "Unable to achieve the desired system clock frequency (with input clock = %d Hz)." % {clkin}
  end
  
  local cpuRate = 100.0/ sysClkHz * 10000000;
  
  io.write("// Clock configurations\n")
  io.write("#define SYSCLK_HZ %iL\n" % {sysClkHz}) 
  io.write("#define LSPCLK_HZ (SYSCLK_HZ / 4l)\n")
    
  io.write("#define PLL_FMULT 0\n")
  io.write("#define PLL_IMULT %i\n" % {imult})
  io.write("#define PLL_SRC " .. clksrc ..  "\n")  
  io.write("#define PLX_CPU_RATE " .. cpuRate .. "L\n")
  io.write("\n")
end

return T
