local T = { }

-- target options
local boards = {'custom', 'launchpad', 'controlcard'} -- must match info.xml file
local uniFlashConfigs = {
    launchpad = 'Launchpad_TMS320F28377S.ccxml',
    controlcard = 'ControlCard_TMS320F28377S.ccxml'
}

function T.GetBoardNameFromComboIndex(index)
    return boards[index]
end

function T.GetUniflashConfig(board)
    return uniFlashConfigs[board]
end

-- resources
local pwmGpio = {}
pwmGpio[1] = {0,1}
pwmGpio[2] = {2,3}
pwmGpio[3] = {4,5}
pwmGpio[4] = {6,7}
pwmGpio[5] = {8,9}
pwmGpio[6] = {10,11}    
pwmGpio[7] = {12,13}
pwmGpio[8] = {14,15}  
pwmGpio[9] = {16,17}
pwmGpio[10] = {18,19}    
pwmGpio[11] = {20,21}
pwmGpio[12] = {22,23} 

local qepPinset = {}
qepPinset["20-21-99"] = 10
qepPinset["10-11-13"] = 11
qepPinset["20-21-13"] = 12
qepPinset["20-11-99"] = 13
qepPinset["20-11-13"] = 14
qepPinset["10-11-99"] = 15
qepPinset["10-21-99"] = 16
qepPinset["10-21-13"] = 17
qepPinset["54-55-57"] = 20
qepPinset["24-25-26"] = 21
qepPinset["62-63-65"] = 30

local spiPinset = {}
spiPinset["58-59-60-61"] = 10
spiPinset["58-59-60"] = 10
spiPinset["16-17-18-19"] = 11
spiPinset["16-17-18"] = 11

spiPinset["63-64-65-66"] = 20
spiPinset["63-64-65"] = 20
spiPinset["24-25-26-27"] = 21
spiPinset["24-25-26"] = 21

local sciPinset = {}
sciPinset["28-29"] = 0
sciPinset["85-84"] = 1
sciPinset["43-42"] = 2

function T.Configure(resources, registry)
  resources:add("Model Trigger", 1, 1)  
  resources:add("Powerstage Control", 1, 1)   
  resources:add("GPIO", 0, 199)    
  resources:add("ADC", 0, 3)   
  resources:add("PWM", 1, 12) 
  resources:add("QEP", 1, 3)   
  resources:add("SPI", 1, 2)
  resources:add("ADCA-SOC", 0, 15)
  resources:add("ADCB-SOC", 0, 15)  
  resources:add("ADCC-SOC", 0, 15)    
  resources:add("ADCD-SOC", 0, 15)
end

function T.GetPwmGpio(unit)
	if pwmGpio[unit]==nil then
	  return -1 -- Error code - resource not available.
	else
	  return pwmGpio[unit]
	end
end

function T.GetSciPinset(pinsetS)
	return sciPinset[pinsetS]
end

function T.GetQepPinset(pinsetS)
	return qepPinset[pinsetS]
end

function T.GetSpiPinset(pinsetS)
	return spiPinset[pinsetS]
end

function T.GetIntOscClock()
	return 10000000 -- INTOSC2
end

function T.GetTimerClock()
    -- to help with margin and latency measurements, timer is configured at rate of PWM
	return T.GetPwmClock()
end

function T.GetPwmClock()
  -- TBCLOCK - CLKDIV=/1, HSPCLKDIV=/2 
  local sysClkHz = Target.Variables.sysClkMHz * 1e6
  if sysClkHz <= 100000000 then
    return sysClkHz / 2  
  else
    return sysClkHz / 4
  end
end

function T.GetDeadTimeClock()
  -- TBCLOCK - CLKDIV=/1, HSPCLKDIV=/2 
  -- TBCLK = EPWMCLK/(HSPCLKDIV * CLKDIV) 
  local sysClkHz = Target.Variables.sysClkMHz * 1e6  
  if sysClkHz <= 100000000 then
    return sysClkHz / 2  
  else
    return sysClkHz / 4
  end
end

function T.GetMaxEventPeriod()
	return 15
end

function T.GetMaxSciBaudRate()
	-- assuming 8N1. 1.5 characters per poll
	local maxRate = 1/Target.Variables.SAMPLE_TIME * 15
	return math.min(115200, maxRate)
end

function T.GetAdcClock()
  -- HAL automatically configures ADCCTL2.bit.PRESCALE
  -- 28377 200MHz system clock
  local sysClkHz = Target.Variables.sysClkMHz * 1e6
  if sysClkHz <= 100000000 then
    return sysClkHz
  else
    return sysClkHz / 2
  end
end

function T.CalcACQPS(ts)
	local tsMin = math.max(75e-9, 1/T.GetAdcClock())-- per datasheet
	if ts ~= ts or ts < tsMin then -- test for NAN or value that is too small
		ts = tsMin 
	end
	local sysClkHz = Target.Variables.sysClkMHz * 1e6
	return math.floor(ts * sysClkHz)
end

function T.GetINTSEL1N2RegisterName()
	return "ADCINTSEL1N2"
end

function T.WriteClockConfiguration(io)
  local sysClkHz = Target.Variables.sysClkMHz * 1e6

  local clkin, clksrc

  if sysClkHz > 200000000 then 
    return "Excessive system clock setting."
  elseif sysClkHz < 2000000 then
    return "Insufficient system clock setting."
  end 
  
  if sysClkHz ~= math.floor(sysClkHz) then
    return "System clock setting must be integer value."
  end 

  local sysClkWcHi = sysClkHz
  if Target.Variables.useIntOsc ~= 1 then
    clkin = Target.Variables.extClkMHz * 1e6
    clksrc = 1
  else
    if sysClkHz > 194000000 then 
      return "Excessive system clock setting for internal oscillator. Must not be greater than 194 MHz."
    end
    sysClkWcHi = 1000000*math.floor((sysClkHz * 1.03+500000)/1000000)
    clkin =  T.GetIntOscClock()
    clksrc = 0
  end
  
  local sysClkDivRegVal = 1 -- hard-coded  
  local sysClkDiv = math.max(1, sysClkDivRegVal*2)

  local fmult = 0 -- TODO: allow different values for more flexibility below
  local fmultRegVal = fmult/0.25
  
  local imult = math.min(math.floor(sysClkDiv*sysClkHz/clkin - fmult), 127)
  local imultRegVal = imult
  
  local pllRawMHz = (imult + fmult) * clkin / 1000000
  if (pllRawMHz < 120) or (pllRawMHz > 400) then
     return "PLL raw frequency (%d MHz) outside of valid range (120-400 MHz)." % {pllRawMHz}
  end
  
  if not (sysClkHz == imult * clkin/sysClkDiv) then
    return "Unable to achieve the desired system clock frequency (with input clock = %d Hz)." % {clkin}
  end
  
  local cpuRate = 100.0/ sysClkHz * 10000000;
  
  io.write("// Clock configurations\n")
  io.write("#define SYSCLK_HZ %iL\n" % {sysClkHz}) 
  io.write("#define SYSCLK_WC_HI_HZ %iL\n" % {sysClkWcHi}) 
  io.write("#define LSPCLK_HZ (SYSCLK_HZ / 4l)\n")
    
  io.write("#define PLL_SYSCLK_DIV %i\n" % {sysClkDivRegVal}) 
  io.write("#define PLL_FMULT %i\n" % {fmultRegVal})
  io.write("#define PLL_IMULT %i\n" % {imultRegVal})
  io.write("#define PLL_SRC " .. clksrc ..  "\n")  
  io.write("#define PLX_CPU_RATE " .. cpuRate .. "L\n")
  io.write("\n")
end

return T
