local C = {}

function C.SetTimerParams(timer, params, T)
    local period = math.max(1, math.floor(T.GetTimerClock()/params["f"]+0.5))
    if period > 0x100000000 then
        return "Unable to achieve the desired timer frequency (%f Hz is too low)." % {params["f"]}
    end    
    
    local achievableF = T.GetTimerClock()/period
    if (params["f_rtol"] ~= nil) and (params["f_atol"] ~= nil) then
        tol = params["f"] * params["f_rtol"]
        if tol < params["f_atol"] then
            tol = params["f_atol"]
        end
        local fError = params["f"] - achievableF
        if math.abs(fError)  > tol then
            local msg = [[
            Unable to accurately achieve the desired timer frequency:
            - desired value: %f Hz
            - closest achievable value: %f Hz
                
            Please modify the frequency setting or change the "Frequency tolerance" parameter.
            You may also adjust the system clock frequency under Coder Options->Target->General.
            ]]
            return msg % {params["f"], achievableF}
        end
    end
    params["f_actual"] = achievableF
    params["prd"] = period
    params["prd_systicks"] = period * Target.Variables.sysClkMHz * 1e6 / T.GetTimerClock()
end

function C.SetPwmHardwareParams(pwm, params, req, T)
    local CTRMODE, prd, AQCAD, AQCAU, AQZRO
    local achievableFsw
    local achievablePeriodInSysTicks
    if params["carrier_type"] == 'triangle' then
        prd = math.floor(T.GetPwmClock()/params["fsw"]/2+0.5)
        periodInSysTicks = prd * 2 * Target.Variables.sysClkMHz * 1e6 / T.GetTimerClock()
        achievableFsw = T.GetPwmClock()/2/prd
        CTRMODE = 2
        AQCAU = 1  --up   = set inactive
        AQCAD = 2  --down = set active
        AQZRO = 2  --zero = set active
        AQPRD = 0  --prd  = no action
    else
        prd = math.floor(T.GetPwmClock()/params["fsw"]-1 + 0.5)
        periodInSysTicks = (prd+1) * Target.Variables.sysClkMHz * 1e6 / T.GetTimerClock()
        achievableFsw = T.GetPwmClock()/(prd+1)
        CTRMODE = 0
        AQCAU = 1 --up   = set inactive
        AQCAD = 0 --down = no action
        AQZRO = 2 --zero = set active
        AQPRD = 0 --prd  = no action
    end

    if prd > 0xFFFF then
        return "Unable to achieve the desired PWM frequency (%f Hz is too low)." % {params["fsw"]}
    end
    
    if (params["fsw_rtol"] ~= nil) and (params["fsw_atol"] ~= nil) then
        tol = params["fsw"] * params["fsw_rtol"]
        if tol < params["fsw_atol"] then
            tol = params["fsw_atol"]
        end
        local fswError = params["fsw"] - achievableFsw
        if math.abs(fswError)  > tol then
            local msg = [[
            Unable to accurately achieve the desired PWM frequency:
            - desired value: %f Hz
            - closest achievable value: %f Hz
                
            Please modify the frequency setting or change the "Frequency tolerance" parameter.
            You may also adjust the system clock frequency under Coder Options->Target->General.
            ]]
            return msg % {params["fsw"], achievableFsw}
        end
    end
    
    params["fsw_actual"] = achievableFsw
    params["prd"] = prd
    params["prd_systicks"] = periodInSysTicks
    params["dt"] = math.floor(params["dead_time"]*T.GetDeadTimeClock())
    params["CTRMODE"] = CTRMODE
    params["AQCAU"] = AQCAU
    params["AQCAD"] = AQCAD
    params["AQZRO"] = AQZRO
    params["AQPRD"] = AQPRD

    -- claim GPIO associated with PWM
    for _, p in ipairs(T.GetPwmGpio(pwm)) do
	if p<0 then
	  return "PWM generator %d is not available for this target device." % {pwm}
        end
        req:add("GPIO", p)
    end
    
    local CBC1, CBC2, CBC3, OSHT1, OSHT2, OSHT3
    if params.tz1_enable == 1 then
        CBC1 = 1
        OSHT1 = 0
    elseif params.tz1_enable == 2 then
        CBC1 = 0
        OSHT1 = 1   
    else
        CBC1 = 0
        OSHT1 = 0
    end
    if params.tz2_enable == 1 then
        CBC2 = 1
        OSHT2 = 0
    elseif params.tz2_enable == 2 then
        CBC2 = 0
        OSHT2 = 1
    else
        CBC2 = 0
        OSHT2 = 0
    end
    if params.tz3_enable == 1 then
        CBC3 = 1
        OSHT3 = 0
    elseif params.tz3_enable == 2 then
        CBC3 = 0
        OSHT3 = 1
    else
        CBC3 = 0
        OSHT3 = 0  
    end
    params.CBC1 = CBC1
    params.CBC2 = CBC2
    params.CBC3 = CBC3
    params.OSHT1 = OSHT1
    params.OSHT2 = OSHT2
    params.OSHT3 = OSHT3
end

function C.SetAdcSocHardwareParams(adc, soc, params, req, T)
    local adcId
    if adc < 0 then
        adcId = 'ADC?'
    else
        adcId = 'ADC%s' % {string.char(65+adc)}
    end

    if soc > 15 then
        return "Maximal number of conversion channels exceeded for %s." % {adcId}
    end

    if params["input"] > 15 then
        return "AIN%i is not a valid input for for %s." % {params["input"], adcId}
    end

    req:add("%s-SOC" % {adcId}, soc)
    params["ACQPS"] = T.CalcACQPS(params["ts"])
end

function C.SetQepHardwareParams(qep, params, req, T)
    local pinsetS
    for _, p in ipairs(params["abi"]) do
      if pinsetS == nil then
        pinsetS = "%i" % {p}
      else
        pinsetS = "%s-%i" % {pinsetS, p}
      end
    end

    local pinset = T.GetQepPinset(pinsetS)
    if pinset == nil then
        return "Qep pinset %s not supported" % {pinsetS}
    end

    params["pinsetS"] = pinsetS
    params["pinset"] = pinset
end

function C.ConfigureQeps(reg, PreInitCode, T)
  for qep, par in pairs(reg.QepConfigs) do
    PreInitCode:append(" // configure QEP%i for pinset %s\n" % {qep, par["pinsetS"]})
    PreInitCode:append("{\n")
    PreInitCode:append("  QEP_Params_t *params = HAL_getDefaultQepParams(%i, %i);\n" % {par["prd"], par["reset_on_index"]})
    PreInitCode:append("  HAL_setupQep(%i, %i, %i, params);\n" % {par["instance"], qep,  par["pinset"]})
    PreInitCode:append("}\n")
  end
end

function C.SetSpiHardwareParams(spi, params, req, T)
    local pinsetS
    for _, p in ipairs(params["pinset"]) do
      if pinsetS == nil then
        pinsetS = "%i" % {p}
      else
        pinsetS = "%s-%i" % {pinsetS, p}
      end
    end

    local pinset = T.GetSpiPinset(pinsetS)
    if pinset == nil then
        return "Spi pinset %s not supported" % {pinsetS}
    end

    params["pinsetS"] = pinsetS
    params["pinset"] = pinset
end

function C.ConfigureSpis(reg, PreInitCode, T)
  for spi, par in pairs(reg.SpiConfigs) do
    PreInitCode:append(" // configure SPI-%s for pinset %s\n" % {string.char(64+spi), par["pinsetS"]})
    PreInitCode:append("{\n")
    PreInitCode:append("  SPI_Params_t *params = HAL_getDefaultSpiParams(%i, %i, %i, %i, %i);\n" % {par["charlen"], par["pol"], par["phase"], par["baudrate"], par["masterslave"]})
    PreInitCode:append("  HAL_setupSpi(%i, %i, %i, params);\n" % {par["instance"], spi,  par["pinset"]})
    PreInitCode:append("}\n")
  end
end

function C.ConfigureTimers(reg, T)
    -- create at least one timer so that it can be used for triggering the ADC and/or model
    if reg.NumTimerBlocks == 0 then
        local f = 1/Target.Variables.SAMPLE_TIME
        local period = math.max(1, math.min(math.floor(T.GetTimerClock()/f+0.5), 0x100000000))
        local periodInSysTicks = period * Target.Variables.sysClkMHz * 1e6 / T.GetTimerClock()
        local params = {
            f = f,
            f_actual = T.GetTimerClock()/period,
            prd = period,
            prd_systicks = periodInSysTicks 
        }
        reg.TimerConfigs[0] = params
    end
end

-- private function
function DetermineAutoTrigger(reg, T, triggerViaAdc)
    local modelClkHz = 1/Target.Variables.SAMPLE_TIME
    
    local autoPwm, autoPwmFreqAbsError
    for pwm, conf in pairs(reg.PwmConfigs) do
        local fevt = conf["fsw_actual"]
        local loc
        if triggerViaAdc then
            loc = conf["soc_loc"]
        else
            loc = conf["int_loc"]
        end
        if (loc ~= nil) and (loc ~= '') then
            fevt = fevt * #loc
        end  
        
        local evtprd = math.max(1, math.min(math.floor(fevt/modelClkHz+0.5), T.GetMaxEventPeriod()))
        local f = fevt/evtprd
              
        local e = math.abs(f - modelClkHz)
        if (autoPwmFreqAbsError == nil) or (autoPwmFreqAbsError > e) then
            autoPwmFreqAbsError = e
            autoPwm = pwm
        end
        if autoPwmFreqAbsError == 0 then
            break -- perfect fit!
        end
    end
    
    if autoPwm ~= nil then
        print("Auto PWM: " .. autoPwm)
        print("PWM abs error [Hz]: " .. autoPwmFreqAbsError)
    end
    
    -- can we do better with a timer?
    local autoTimer, autoTimerFreqAbsError
    for timer, conf in pairs(reg.TimerConfigs) do
        local f = conf["f_actual"]
        
        local e = math.abs(f - modelClkHz)
        if (autoTimerFreqAbsError == nil) or (autoTimerFreqAbsError > e) then
            autoTimerFreqAbsError = e
            autoTimer = timer
        end
        if autoTimerFreqAbsError == 0 then
            break -- perfect fit!
        end
    end
    
    if autoTimer ~= nil then
        print("Auto Timer: " .. autoTimer)
        print("Timer abs error [Hz]: " .. autoTimerFreqAbsError)
    end
    
    if (autoPwm ~= nil) and (autoTimer ~= nil) then
        -- retain the more accurate option
        if autoTimerFreqAbsError < autoPwmFreqAbsError then
            autoPwm = nil -- use timer
        end
    end
    return {pwm = autoPwm, timer = autoTimer}
end

function C.ConfigureAdcSocTriggers(reg, T)
    if reg.NumAdcBlocks == 0 then
        return
    end
    
    -- all SOC have to be either trigger explicitly or implicitly
    local numAutoTriggers = 0
    local numExplicitTriggers = 0
    local autoTrigger = DetermineAutoTrigger(reg, T, true)
    for adc, v in pairs(reg.AdcConfigs) do
        local va = reg.AdcSocConfigs[adc]
        for soc, vs in pairs(va) do
            local trig = vs["trig"]
            local type = trig[1]["type"]
            if type == "auto" then
                numAutoTriggers = numAutoTriggers + 1
            else
                numExplicitTriggers = numExplicitTriggers + 1
            end
        end
    end
    
    -- a mixed triggering mode can create ambiguities and unexpected behavior
    if (numAutoTriggers ~= 0) and (numExplicitTriggers ~= 0) then
        local msg = [[
        Explicitely connect all ADCs with an ADC trigger signal, or leave them all unconnected.
        A mix of the two options is not allowed.
        ]]
        return msg
    end
    
    -- when the ADCs are explicitly triggered, also require an explicit model trigger
     if (numExplicitTriggers ~= 0) then
         if (reg.ModelTrigger == nil) or (reg.ModelTrigger[1]["type"] == "auto") then
            local msg = [[
            Models with explicitely triggered ADC blocks also require the use of a Control Task Trigger block.
            ]]
            return msg
          end  
     end
    
    -- when ADCs are implicitly triggered, use PWM or timer that yields the best accuracy
    if (numAutoTriggers ~= 0) then
        -- determine auto trigger source
        local autoTrigger = DetermineAutoTrigger(reg, T, true)
        local trig
        if autoTrigger["pwm"] ~= nil then
            local autoPwm = autoTrigger["pwm"]
            trig = {{type = "pwm", unit = autoPwm}}
            -- see if we need to automatically enable the SOC trigger event
            local socLoc = reg.PwmConfigs[autoPwm]["soc_loc"]
            if (socLoc == nil) or (socLoc == "") then
                -- auto-configure to trigger SOC at zero
                 reg.PwmConfigs[autoPwm]["soc_loc"] = "z"
            end
        elseif autoTrigger["timer"] ~= nil then
            trig = {{type = "timer", unit = autoTrigger["timer"]}}
        else
            return "Unable to determine SOC trigger. This is a bug."
        end
        print("SOC auto trigger: " .. dump(trig))
        -- configure all SOCs in all ADCs
        for adc, v in pairs(reg.AdcConfigs) do
           local va = reg.AdcSocConfigs[adc]
           for soc, vs in pairs(va) do
               vs["trig"] = trig
           end
        end
    end
end

function C.ConfigurePwms(reg, PreInitCode, T)
    if reg.NumPwmBlocks == 0 then
        return
    end

    local sortedPwmArray = {}
    for pwm in pairs(reg.PwmConfigs) do table.insert(sortedPwmArray, pwm) end
    table.sort(sortedPwmArray)
    for _, pwm in ipairs(sortedPwmArray) do
        local par = reg.PwmConfigs[pwm]
        local SOCASEL
        if par["soc_loc"] == 'z' then
            SOCASEL = 1
        elseif par["soc_loc"] == 'p' then
            SOCASEL = 2
        elseif par["soc_loc"] == 'zp' then
            SOCASEL = 3
        end

        PreInitCode:append(" // configure PWM%i at %.1f Hz in %s mode (soc='%s')\n" % {pwm, par["fsw_actual"], par["carrier_type"], par["soc_loc"]})
        if par["fsw_actual"] ~= par["fsw"] then
            PreInitCode:append(" // (desired frequency was %1.f Hz)\n" % {par["fsw"]})
        end
        PreInitCode:append("{\n")
        local dummy = 0
        PreInitCode:append("  PWM_Params_t *params = HAL_getDefaultPwmParams(%i, %i, %i);\n" % {par["prd"], par["dt"], dummy})
        PreInitCode:append("  params->TBCTL.bit.CTRMODE = %i;\n" % {par["CTRMODE"]})
        if par["polarity"] > 0 then
            PreInitCode:append("  // active high complementary\n")
            PreInitCode:append("  params->DBCTL.bit.POLSEL = 2;\n")
        else
            PreInitCode:append("  // active low complementary\n")
            PreInitCode:append("  params->DBCTL.bit.POLSEL = 1;\n")
        end
        PreInitCode:append("  params->AQCTLA.bit.CAU = %i;\n" % {par["AQCAU"]})
        PreInitCode:append("  params->AQCTLA.bit.CAD = %i;\n" % {par["AQCAD"]})
        PreInitCode:append("  params->AQCTLA.bit.ZRO = %i;\n" % {par["AQZRO"]})
        PreInitCode:append("  params->AQCTLA.bit.PRD = %i;\n" % {par["AQPRD"]})
        if SOCASEL ~= nil then
            PreInitCode:append("  params->ETSEL.bit.SOCASEL = %i;\n" % {SOCASEL})
        end
        PreInitCode:append("  params->TZSEL.bit.CBC1 = %i;\n" % {par["CBC1"]})
        PreInitCode:append("  params->TZSEL.bit.CBC2 = %i;\n" % {par["CBC2"]})
        PreInitCode:append("  params->TZSEL.bit.CBC3 = %i;\n" % {par["CBC3"]})
        PreInitCode:append("  params->TZSEL.bit.OSHT1 = %i;\n" % {par["OSHT1"]})
        PreInitCode:append("  params->TZSEL.bit.OSHT2 = %i;\n" % {par["OSHT2"]})
        PreInitCode:append("  params->TZSEL.bit.OSHT3 = %i;\n" % {par["OSHT3"]})
        if reg.PowerstageProtectionConfig ~= nil then
            local tzsafe
            if reg.PowerstageProtectionConfig["forcesafe"] then
               if par["polarity"] > 0 then
                   PreInitCode:append("  // force low when tripped\n")
                   tzsafe = 2
               else
                   PreInitCode:append("  // force high when tripped\n")
                   tzsafe = 1
               end 
            else
                PreInitCode:append("  // float output when tripped\n")
                tzsafe = 0
            end
            PreInitCode:append("  params->TZCTL.bit.TZA = %i;\n" % {tzsafe})
            PreInitCode:append("  params->TZCTL.bit.TZB = %i;\n" % {tzsafe})
        end
        
        if par["phsen"] ~= nil and par["syncosel"] ~= nil then
          PreInitCode:append("  params->TBCTL.bit.PHSEN = %i;\n" % {par["phsen"]})
          PreInitCode:append("  params->TBCTL.bit.SYNCOSEL = %i;\n" % {par["syncosel"]})  
        end
        
        if reg.PwmSyncConfigs[pwm] ~= nil then
          if reg.PwmSyncConfigs[pwm]["type"] == 'pwm' then
            PreInitCode:append("  HAL_setupPwmSyncSelect(%i, %i, %i);\n"% {pwm, 0, reg.PwmSyncConfigs[pwm]["unit"]}) 
          elseif reg.PwmSyncConfigs[pwm]["type"] == 'gpio' then
            PreInitCode:append("  HAL_setupPwmSyncSelect(%i, %i, %i);\n"% {pwm, 1, reg.PwmSyncConfigs[pwm]["unit"]}) 
          else
          end 
        end
        
        PreInitCode:append("  HAL_setupPwm(%i, %i, params);\n"% {par["instance"], pwm})
        PreInitCode:append("}\n")
    end
end

function C.ConfigureAdcs(reg, PreInitCode, T)
    if reg.NumAdcBlocks == 0 then
        return
    end
    local adc_int, INT1SEL
    if reg.ModelTrigger[1]["type"] == 'adc' then
        adc_int =  reg.ModelTrigger[1]["unit"]
        INT1SEL = reg.ModelTrigger[1]["loc"]
    end

    local sortedAdcArray = {}
    for adc in pairs(reg.AdcConfigs) do table.insert(sortedAdcArray, adc) end
    table.sort(sortedAdcArray)
    for _, adc in ipairs(sortedAdcArray) do
        local v = reg.AdcConfigs[adc]
        PreInitCode:append(" // configure ADC %s\n" % {string.char(65+adc)})
        PreInitCode:append("{\n")
        PreInitCode:append("  AIN_AdcParams_t *params = HAL_getDefaultAdcParams();\n")
        if adc == adc_int then
            PreInitCode:append(" // configure SOC%i to trigger interrupt\n" % {INT1SEL})
            PreInitCode:append("  params->%s.bit.INT1SEL = %i;\n" % {T.GetINTSEL1N2RegisterName(), INT1SEL})
        end
        PreInitCode:append("  HAL_setupAdc(%i, %i, params);\n" %  {v["channel"], adc})
        PreInitCode:append("}\n")

        local sortedSocArray = {}
        for soc in pairs(reg.AdcSocConfigs[adc]) do table.insert(sortedSocArray, soc) end
        table.sort(sortedSocArray)
        for _, soc in ipairs(sortedSocArray) do
            local vs = reg.AdcSocConfigs[adc][soc]
            local trig = vs["trig"]
            local type = trig[1]["type"]
            local unit = trig[1]["unit"]

            local TRIGSEL, trigSelText
            if type == 'pwm' then
                pwmconf = reg.PwmConfigs[unit]
                if pwmconf == nil then
                    return "PWM%i not configured and therefore unavailable as ADC SOC trigger" % {unit}
                end
                TRIGSEL = 5 + 2*(unit-1)
                trigSelText = "PWM%i" % {unit}
            else
                -- default is timer 0
                TRIGSEL = 1
                trigSelText = "CPU Timer%i" % {0}
            end

            PreInitCode:append(" // configure SOC%i of ADC-%s to measure ADCIN%i\n" % {soc, string.char(65+adc), vs["input"]})
            PreInitCode:append("{\n")
            if type == 'auto' then
                PreInitCode:append(" // SOC trigger for this channel has been automatically selected\n")
            end
            PreInitCode:append("  AIN_ChannelParams_t *paramsChannel = HAL_getDefaultAinChannelParams();\n")
            PreInitCode:append("  // set SOC trigger to %s\n" % {trigSelText})
            PreInitCode:append("  paramsChannel->ADCSOCxCTL.bit.TRIGSEL = %i;\n" % {TRIGSEL})
            PreInitCode:append("  paramsChannel->ADCSOCxCTL.bit.ACQPS = %i;\n" % {vs["ACQPS"]})
            PreInitCode:append("  HAL_setupAnalogInF(%i, %i, %i, paramsChannel, %.9ef, %.9ef);\n" % {vs["instance"], soc, vs["input"], vs["scale"], vs["offset"]})
            PreInitCode:append("}\n")
        end
    end
end

function C.ConfigureModelTrigger(reg, T)
    if (reg.ModelTrigger ~= nil) and (reg.ModelTrigger[1]["type"] ~= "auto") then
        return
    end
    
    -- perform an automatic model trigger configuration
    reg.ModelTrigger = nil
    
    if reg.NumAdcBlocks ~= 0 then
        -- we have ADC blocks in the model and they will be used for auto-triggering (=implicit convention)
        -- need to find ADC with most SOC - Note that we are assuming that the sampling
        -- time is set to be identical, and hence we have overlapping synchronous conversions,
        -- as recommended by TI
        local adcTrig, socTrig
        local maxNumSocs = 0
        for adc, adcConf in pairs(reg.AdcConfigs) do
            local va = reg.AdcSocConfigs[adc]
            local thisHighestSoc = 0
            local thisNumSoc = 0
            for soc, _ in pairs(va) do
                thisNumSoc = thisNumSoc + 1
                if thisHighestSoc < soc then
                    thisHighestSoc = soc
                end
            end
            if maxNumSocs < thisNumSoc then
                maxNumSocs = thisNumSoc
                adcTrig = adc
                socTrig = thisHighestSoc
            end
        end
        reg.ModelTrigger = {{type = "adc", unit = adcTrig, loc = "%i" % {socTrig}}}
    else
        -- for models w/o ADC we use the best fitting PWM or timer
        local autoTrigger = DetermineAutoTrigger(reg, T, false)
        if autoTrigger["pwm"] ~= nil then
            reg.ModelTrigger = {{type = "pwm", unit = autoTrigger["pwm"]}}
        elseif autoTrigger["timer"] ~= nil then
            reg.ModelTrigger = {{type = "timer", unit = autoTrigger["timer"]}}
        else
            return "Unable to determine model trigger. This is a bug."
        end
    end
    print("Model auto trigger: " .. dump(reg.ModelTrigger))
end

function C.WriteDispatcherConfiguration(io, reg, T)
    local modelClkHz = math.floor(1/Target.Variables.SAMPLE_TIME+0.5)
        
    io.write("// Dispatcher configurations\n")

    local triggerType = reg.ModelTrigger[1]["type"]
    local triggerUnit = reg.ModelTrigger[1]["unit"]
    local triggerLoc = reg.ModelTrigger[1]["loc"]
        
    -- determine DISP_CONFIGURE_TRIGGER() and achievable model period
    local desiredPeriodInSystTicks = math.floor(Target.Variables.sysClkMHz * 1e6 * Target.Variables.SAMPLE_TIME + 0.5)
    local achievableModelPeriodInSysClk  
    if triggerType == "timer" then
        achievableModelPeriodInSysClk = reg.TimerConfigs[triggerUnit]["prd_systicks"]
        io.write("#define DISP_CONFIGURE_TRIGGER() DISPR_setTriggerByTimer()\n")
        
    elseif triggerType == 'adc' then
        -- indirect triggering through ADC
        local conf = reg.AdcSocConfigs[triggerUnit]
        if conf == nil then
            return string.format("ADC%i not configured and therefore not available as time base clock.", triggerUnit)
        end

        local socconf = conf[tonumber(triggerLoc)]
        if socconf == nil then
            return string.format("SOC%i of ADC%i not configured and therefore not available as time base clock.", triggerLoc, triggerUnit)
        end

        local soctrig = socconf["trig"]
        if soctrig[1]["type"] == "pwm" then
            -- PWM triggers ADC
            local pwm = soctrig[1]["unit"]
            -- this SOC is triggered by PWM
            local conf = reg.PwmConfigs[pwm]
            local prdInSysTicks = conf["prd_systicks"]
            local socLoc = conf["soc_loc"]
            if (socLoc ~= nil) and (socLoc ~= '') then
                prdInSysTicks = prdInSysTicks / #socLoc
            end
            local evtprd = math.max(1, math.min(math.floor(desiredPeriodInSystTicks/prdInSysTicks+0.5), T.GetMaxEventPeriod()))
            achievableModelPeriodInSysClk = evtprd * prdInSysTicks
            io.write("#define DISP_CONFIGURE_TRIGGER()  DISPR_setTriggerByAdcViaPwm((AIN_Unit_t)%i, %i, (PWM_Unit_t)%i)\n" % {triggerUnit, triggerLoc, pwm})
        elseif soctrig[1]["type"] == "timer" then
            local timer = soctrig[1]["unit"]                                          
            achievableModelPeriodInSysClk = reg.TimerConfigs[timer]["prd_systicks"]
            io.write("#define DISP_CONFIGURE_TRIGGER()  DISPR_setTriggerByAdcViaTimer((AIN_Unit_t)%i, %i)\n" % {triggerUnit, triggerLoc})
        else
            return "Implementation error. Unexpected SOC trigger type: %s" % {soctrig[1]["type"]}
        end

    elseif triggerType == 'pwm' then
        -- direct triggering via PWM
        local conf = reg.PwmConfigs[triggerUnit]
        local prdInSysTicks = conf["prd_systicks"]

        if (triggerLoc ~= nil) and (triggerLoc ~= '') then
            prdInSysTicks = prdInSysTicks / #triggerLoc
        end

        local evtprd = math.max(1, math.min(math.floor(desiredPeriodInSystTicks/prdInSysTicks+0.5), T.GetMaxEventPeriod()))
        achievableModelPeriodInSysClk = evtprd * prdInSysTicks
            
        local INTSEL
        if triggerLoc == 'p' then
            INTSEL = 2
        elseif triggerLoc == 'zp' then
            INTSEL = 3
        else
            INTSEL = 1 -- default value for ETSEL.bit.INTSEL is ET_CTR_ZERO
        end
        io.write("#define DISP_CONFIGURE_TRIGGER() DISPR_setTriggerByPwm((PWM_Unit_t)%i, %i)\n" % {triggerUnit, INTSEL})
    else
        return "Implementation error. Unexpected trigger type: %s" % {triggerType}
    end
        
    -- final tolerance check
    local achievableModelClkHz = Target.Variables.sysClkMHz * 1e6 / achievableModelPeriodInSysClk
    
    print("Achievable Model Clock Hz:" .. achievableModelClkHz)
    
    local taskFreqErrorHz = (achievableModelClkHz - modelClkHz)
    if Target.Variables.taskFreqTol == 1 then
        tol = 1e-6 * modelClkHz
        if math.abs(taskFreqErrorHz) > tol then
            local msg = [[
            Unable to accurately achieve the desired step size:
            - desired value: %e
            - closest achievable value: %e

            You may want to change the "Step size tolerance" parameter or adjust the system clock frequency
            under Coder Options->Target->General.
            ]]
            return msg % {Target.Variables.SAMPLE_TIME, 1/achievableModelClkHz}
        end
    end
    
    if (math.floor(modelClkHz/Target.Variables.task2Hz+0.5) - (modelClkHz/Target.Variables.task2Hz)) > 4e-9 then 
	-- check to see if integer divisor with tolerance ~eps(25e6)
        local msg = [[
        Control task frequency must be an exact multiple of the infrastructure task frequency.
        - control task frequency: %f Hz
        - infrastructure task frequency: %f Hz
        ]]
        return msg % {modelClkHz, Target.Variables.task2Hz}
    end
    
    local achievableTask2PeriodInSysClk = achievableModelPeriodInSysClk * math.floor(achievableModelClkHz/Target.Variables.task2Hz + 0.5)
    
    io.write("#define TASK1_HZ %i // exact value: %e Hz\n" % {modelClkHz, (Target.Variables.sysClkMHz * 1e6)/achievableModelPeriodInSysClk})
    io.write("#define TASK1_PRD_IN_SYS_CLK_TICKS %iL\n" % {achievableModelPeriodInSysClk})
    io.write("#define TASK2_HZ %i // exact value: %e Hz\n" % {Target.Variables.task2Hz, (Target.Variables.sysClkMHz * 1e6)/achievableTask2PeriodInSysClk})
    io.write("#define TASK2_PRD_IN_SYS_CLK_TICKS %iL\n" % {achievableTask2PeriodInSysClk})
    io.write("\n")
end

return C
